#ifndef __GS_AR_TRAINER_H__
#define __GS_AR_TRAINER_H__

#include <stdio.h>
#include <stdlib.h>
#include "projects\COMMS.h"

/*Defines for each of the commands*/
#define TR_READ		1
#define TR_WRITE		2
#define TR_ADD_CODE		'i'
#define TR_NUM_CODE		'j'
#define TR_SUB_CODE		'k'
#define TR_PRINT_CODES	'p'

/*returns from the Add Active Code feature*/
#define AAC_LIMIT	0xFE
#define AAC_FOUND	0xFF
#define AAC_OK	0

/*might want to make a more generic stream version*/
int UploadFile(FILE * infile, unsigned long address);
int DownloadFile(FILE * infile, unsigned long size, unsigned long address);

/*these are simple read/write types that do not initiate or close the comms
between read/write, send address & length 0 and reinitiate*/
unsigned long Read32(unsigned long address);
unsigned short Read16(unsigned long address);
unsigned char ReadByte(unsigned long address);
int Write32(unsigned long address, unsigned long value);
int Write16(unsigned long address, unsigned short value);
int WriteByte(unsigned long address, unsigned char value);

/*Gameshark Code Generator List functions*/
char AddActiveCode(unsigned long address, unsigned long data);
char NumActiveCodes(void);
int  RubActiveCode(unsigned long address);
int  EraseAllCodes(void);
int  PrintActiveCodes(FILE * infile);
/*int screenshot(FILE * image);*/
int GetCodeState(void);
int SetCodeState(int mode);	/*0-on, 1-off, -1-toggle; returns new state*/


/*****************************************
Generic comms functions courtesy of hcs
http://sourceforge.net/projects/N64dev
nifty, eh?
*****************************************/

/*Open for reading/writing*/
#define InitRead() SendCommand(TR_READ)
#define InitWrite() SendCommand(TR_WRITE)

/*these are simple read/write types as above, but don't initiate or close the comms
  These are slightly faster, but the others are more robust*/
unsigned long Read32(unsigned long address) {
	ReadWrite32(address);
	ReadWrite32(4);
	return ReadWrite32(0);
}


unsigned short Read16(unsigned long address) {
	ReadWrite32(address);
	ReadWrite32(2);
	return ReadWrite16(0);
}

unsigned char ReadByte(unsigned long address)
{ReadWrite32(address);
ReadWrite32(1);
return ReadWriteByte(0);
}


int Write32(unsigned long address,unsigned long value) {
	ReadWrite32(address);
	ReadWrite32(4);
	ReadWrite32(value);
	return 0;
}

int Write16(unsigned long address,unsigned short value) {
	ReadWrite32(address);
	ReadWrite32(2);
	ReadWrite16(value);
	return 0;
}

int WriteByte(unsigned long address, unsigned char value) {
	ReadWrite32(address);
	ReadWrite32(1);
	ReadWriteByte(value);
	return 0;
}

/*File Upload*/
int UploadFile(FILE * infile, unsigned long address) {
	unsigned long c=0;
	unsigned long size;
	char buf,x=0;

	if (fseek(infile,0,SEEK_END)) return 1;
	size=ftell(infile);
	rewind(infile);

	if (InitWrite()) return 1;

	ReadWrite32(address);
	ReadWrite32(size);
	printf("Uploading... 0%%");
	
	for (c=0; c < size; c++) {
		fread(&buf,1,1,infile);
		if (c & 0x400) printf("\b\b\b%2d%%",c*100/size);
		ReadWriteByte(buf);
		x+=buf;
	}

	ReadWrite32(0);
	if(x!=ReadWriteByte(0)) printf("\b\b\bChecksum invalid!\n");
	else printf("\b\b\bDone.\n");
	

	/*if (!CloseGSComms()) return 1;*/
	return 0;
}

/*File Download*/
int DownloadFile(FILE * infile, unsigned long size, unsigned long address) {
	unsigned long c=0;
	unsigned char x=0,y=0;
	
	if (InitRead()) return 1;

	ReadWrite32(address);
	ReadWrite32(size);
	printf("Writing to file... 0%%");

	for (c=0; c < size; c++) {
	 y=ReadWriteByte(0);
        fputc(y,infile);
	 x+=y;
        if (c & 0x400) printf("\b\b\b%2d%%",c*100/size);}

	ReadWrite32(0);
	if(x!=ReadWriteByte(0)) printf("\b\b\bChecksum invalid!\n");
	else printf("\b\b\bDone.\n");
	
	/*if (!CloseGSComms()) return 1;*/
	return 0;}

/*add active code to active code list
  note that data is sent as a word, though actual data is a short*/
char AddActiveCode(unsigned long address, unsigned long data) {
	if(SendCommand(TR_ADD_CODE)) return -1;
	ReadWrite32(address);
	ReadWrite32(data);
	return ReadWriteByte(0);
}

/*these are defined as macros, though you really don't need to*/
char NumActiveCodes(void) {
	if(SendCommand(TR_NUM_CODE)) return -1;
	return ReadWriteByte(0);
}

int RubActiveCode(unsigned long address) {
	if(SendCommand(TR_SUB_CODE)) return -1;
	ReadWrite32(address);
	return 0;
	}

/*erases all active codes from list, because man it's iritating otherwise*/
int EraseAllCodes(void) {
	if(InitWrite()) return -1;
	Write32(0xA07E9D60,0);
	ReadWrite32(0);
	return ReadWriteByte(0);
}


/*Prints active codelist as binary; returns number of codes
  might want a more generic stream vresion of this*/
int PrintActiveCodes(FILE * infile){
	unsigned long c=0,x,y,z;
	
	if(SendCommand(TR_PRINT_CODES)) return -1;
	
	c=ReadWrite32(0);
	/*leave it to caller for general init. and command*/
	for(z=c;c>0;c--){
		x=ReadWrite32(0);
		y=ReadWrite32(0);
		fwrite(&x,4,1,infile);
		fwrite(&y,4,1,infile);
		}
	return z;
}

/*TODO: int screenshot(FILE * image)*/

/*used to return state of codelist
  0-on, 1-off.  -1 is a read error*/
int GetCodeState(void){
int mode;
if (InitRead()) return -1;
	mode=Read32(0xA07E9C04);
	ReadWrite32(0);
	ReadWriteByte(0);
return mode;}

/*Sets state of codelist 0-on, 1-off, -1-toggle current mode
  returns the new state or -1 if an error occured*/
int SetCodeState(int mode){
if(mode<0 | mode>1)
	{mode=GetCodeState();
	if(mode<0) return -1;
	mode^=1;
	}

if (InitWrite()) return -1;
	Write32(0xA07E9C04,mode);
	ReadWrite32(0);
	ReadWriteByte(0);

return GetCodeState();}

#endif

